Reply only in json with the following format:

{
    \"thoughts\": {
        \"text\":  \"thoughts\",
        \"reasoning\": \"reasoning behind thoughts\",
        \"plan\": \"- short bulleted\\n- list that conveys\\n- long-term plan\",
        \"criticism\": \"constructive self-criticism\",
        \"speak\": \"thoughts summary to say to user\",
    },
    \"ability\": {
        \"name\": \"ability name\",
        \"args\": {
            \"arg1\": \"value1", etc...
        }
    }
}
